searchData={"items":[{"type":"module","title":"Membership","doc":"Main Membership module for including macros\n\nMembership has 4 main components:\n\n  * `Membership.Role` -  Representation of a single role e.g. :admin, :luser, :user\n  * `Membership.Plan` -  Representation of a single plan e.g. :gold, :silver, :copper\n  * `Membership.Member` - Main actor which is holding given plans\n  * `Membership.Feature` - Feature of a plan eg. :edit_feature","ref":"Membership.html"},{"type":"module","title":"Relations between models - Membership","doc":"`Membership.Member`  -> `Membership.Feature`[1-n] - Any given member have multiple features with this we can have more granular features for each member is adding a specific feature to a member not in his plan\n\n`Membership.Member` -> `Membership.Plan` [1-n] - Any given member can have multiple plans\n\n`Membership.Member` -> `Membership.Role` [1-n] - Any given member can have multiple roles\n\n`Membership.Plan` -> `Membership.Feature` [m-n] - Any given plan can have multiple features\n\n`Membership.Role` -> `Membership.Feature` [m-n] - Any given role can have multiple features","ref":"Membership.html#module-relations-between-models"},{"type":"module","title":"Calculating permissions - Membership","doc":"Calculation of permissions is done by 2 ets tables one holding the logged in members permissions the other holds modules function/permissions, then\n  true = Enum.member?(module_permissions, member_permissions)","ref":"Membership.html#module-calculating-permissions"},{"type":"module","title":"Available as_authorized - Membership","doc":"* `Membership.has_plan?/3` - Requires single plan to be present on member\n  * `Membership.has_role?/3` - Requires single role to be present on member\n  * `Membership.has_feature?/3` - Requires single feature to be present on member","ref":"Membership.html#module-available-as_authorized"},{"type":"function","title":"Membership.add_function_param_to_block/1","doc":"","ref":"Membership.html#add_function_param_to_block/1"},{"type":"macro","title":"Membership.as_authorized/3","doc":"Macro for wrapping protected code","ref":"Membership.html#as_authorized/3"},{"type":"macro","title":"Example - Membership.as_authorized/3","doc":"defmodule HelloTest do\n      use Membership\n      member = HelloTest.Repo.get(Membership.Member, 1)\n      {:ok, member }  = load_and_authorize_member(member)\n\n      def test_authorization do\n        as_authorized(member, :test_authorization) do\n          IO.inspect(\"This code is executed only for authorized member\")\n        end\n      end\n    end","ref":"Membership.html#as_authorized/3-example"},{"type":"function","title":"Membership.authorized?/2","doc":"Returns authorization result on collected member and required features/plans","ref":"Membership.html#authorized?/2"},{"type":"function","title":"Example - Membership.authorized?/2","doc":"defmodule HelloTest do\n      use Membership\n\n      def test_authorization do\n        case authorized? do\n          :ok -> \"Member is authorized\"\n          {:error, message: _message} -> \"Member is not authorized\"\n      end\n    end\n    end","ref":"Membership.html#authorized?/2-example"},{"type":"macro","title":"Membership.calculated/2","doc":"Defines calculated permission to be evaluated in runtime","ref":"Membership.html#calculated/2"},{"type":"macro","title":"Examples - Membership.calculated/2","doc":"defmodule HelloTest do\n      use Membership\n      member = HelloTest.Repo.get(Membership.Member, 1)\n      {:ok, member }  = load_and_authorize_member(member)\n\n      def test_authorization do\n        permissions do\n          calculated(fn member ->\n            member.email_confirmed?\n          end)\n        end\n\n        as_authorized(member) do\n          IO.inspect(\"This code is executed only for authorized member\")\n        end\n      end\n    end\n\nYou can also use DSL form which takes function name as argument\n\n      defmodule HelloTest do\n      use Membership\n\n      def test_authorization do\n      use Membership\n      member = HelloTest.Repo.get(Membership.Member, 1)\n     {:ok, member } = load_and_authorize_member(member)\n\n        permissions do\n          calculated(member,:email_confirmed, :calculated)\n        end\n\n        as_authorized(member) do\n          IO.inspect(\"This code is executed only for authorized member\")\n        end\n      end\n\n      def email_confirmed(member) do\n        member.email_confirmed?\n      end\n    end\n\n  For more complex calculations you need to pass bindings to the function\n\n      defmodule HelloTest do\n      use Membership\n      member = HelloTest.Repo.get(Membership.Member, 1)\n      {:ok, member} = load_and_authorize_member(member)\n\n      def test_authorization do\n        post = %Post{owner_id: 1}\n\n        permissions do\n          calculated(member,:is_owner, [post])\n          calculated(fn member, [post] ->\n            post.owner_id == member.id\n          end)\n        end\n\n        as_authorized(member) do\n          IO.inspect(\"This code is executed only for authorized member\")\n        end\n      end\n\n      def is_owner(member, [post]) do\n        post.owner_id == member.id\n      end\n    end","ref":"Membership.html#calculated/2-examples"},{"type":"macro","title":"Membership.calculated/3","doc":"","ref":"Membership.html#calculated/3"},{"type":"macro","title":"Membership.calculated/4","doc":"","ref":"Membership.html#calculated/4"},{"type":"function","title":"Membership.config/1","doc":"Looks up `Application` config or raises if keyspace is not configured.","ref":"Membership.html#config/1"},{"type":"function","title":"Examples - Membership.config/1","doc":"config :membership, :files, [\n      uploads_dir: Path.expand(\"../priv/uploads\", __DIR__),\n      host: [scheme: \"http\", host: \"localhost\", port: 4000],\n    ]\n\n    iex> Membership.config([:files, :uploads_dir])\n    iex> Membership.config([:files, :host, :port])","ref":"Membership.html#config/1-examples"},{"type":"function","title":"Membership.get_loaded_modules/0","doc":"List membership and related modules.","ref":"Membership.html#get_loaded_modules/0"},{"type":"function","title":"Examples - Membership.get_loaded_modules/0","doc":"iex> Membership.get_loaded_modules()","ref":"Membership.html#get_loaded_modules/0-examples"},{"type":"function","title":"Membership.has_feature/2","doc":"Requires a feature within permissions block","ref":"Membership.html#has_feature/2"},{"type":"function","title":"Example - Membership.has_feature/2","doc":"defmodule HelloTest do\n      use Membership\n\n      def test_authorization do\n        permissions do\n          has_feature(:admin)\n        end\n      end\n    end","ref":"Membership.html#has_feature/2-example"},{"type":"function","title":"Membership.has_feature?/2","doc":"Perform feature check on passed member and feature","ref":"Membership.html#has_feature?/2"},{"type":"function","title":"Membership.has_feature?/3","doc":"","ref":"Membership.html#has_feature?/3"},{"type":"function","title":"Membership.has_plan/2","doc":"Requires an plan within permissions block","ref":"Membership.html#has_plan/2"},{"type":"function","title":"Example - Membership.has_plan/2","doc":"defmodule HelloTest do\n      use Membership\n\n      def test_authorization do\n        permissions do\n          has_plan(:gold, :test_authorization)\n        end\n      end\n    end","ref":"Membership.html#has_plan/2-example"},{"type":"function","title":"Membership.has_plan?/3","doc":"Perform authorization on passed member and plans","ref":"Membership.html#has_plan?/3"},{"type":"function","title":"Membership.has_role/2","doc":"Requires an role within permissions block","ref":"Membership.html#has_role/2"},{"type":"function","title":"Example - Membership.has_role/2","doc":"defmodule HelloTest do\n      use Membership\n\n      def test_authorization do\n        permissions do\n          has_role(:gold, :test_authorization)\n        end\n      end\n    end","ref":"Membership.html#has_role/2-example"},{"type":"function","title":"Membership.has_role?/2","doc":"Perform authorization on passed member and roles","ref":"Membership.html#has_role?/2"},{"type":"function","title":"Membership.has_role?/3","doc":"Perform authorization on passed member and roles","ref":"Membership.html#has_role?/3"},{"type":"function","title":"Membership.ignored_functions/0","doc":"The Function list to ignore when building the permissions registry's","ref":"Membership.html#ignored_functions/0"},{"type":"function","title":"Membership.load_ets_data/1","doc":"Load the plans into ets for the module/functions","ref":"Membership.html#load_ets_data/1"},{"type":"macro","title":"Membership.permissions/1","doc":"Macro for defining required permissions","ref":"Membership.html#permissions/1"},{"type":"macro","title":"Example - Membership.permissions/1","doc":"defmodule HelloTest do\n      use Membership\n\n      def test_authorization do\n        permissions do\n          has_feature(:admin_feature, :test_authorization)\n          has_plan(:gold, :test_authorization)\n        end\n      end\n    end","ref":"Membership.html#permissions/1-example"},{"type":"macro","title":"Membership.permissions/2","doc":"","ref":"Membership.html#permissions/2"},{"type":"function","title":"Membership.version/0","doc":"List version.","ref":"Membership.html#version/0"},{"type":"function","title":"Examples - Membership.version/0","doc":"iex> Membership.version()","ref":"Membership.html#version/0-examples"},{"type":"module","title":"Membership.Behaviour","doc":"","ref":"Membership.Behaviour.html"},{"type":"module","title":"Membership.Config","doc":"","ref":"Membership.Config.html"},{"type":"function","title":"Membership.Config.config/0","doc":"","ref":"Membership.Config.html#config/0"},{"type":"function","title":"Membership.Config.get/2","doc":"Return value by key from config.exs file.","ref":"Membership.Config.html#get/2"},{"type":"function","title":"Membership.Config.key_type/0","doc":"","ref":"Membership.Config.html#key_type/0"},{"type":"function","title":"Membership.Config.key_type/1","doc":"","ref":"Membership.Config.html#key_type/1"},{"type":"function","title":"Membership.Config.repo/0","doc":"","ref":"Membership.Config.html#repo/0"},{"type":"function","title":"Membership.Config.repos/0","doc":"","ref":"Membership.Config.html#repos/0"},{"type":"module","title":"Membership.Feature","doc":"Feature is main representation of a single feature flag assigned to a plan","ref":"Membership.Feature.html"},{"type":"function","title":"Membership.Feature.build/2","doc":"","ref":"Membership.Feature.html#build/2"},{"type":"function","title":"Membership.Feature.changeset/2","doc":"","ref":"Membership.Feature.html#changeset/2"},{"type":"function","title":"Membership.Feature.create/2","doc":"","ref":"Membership.Feature.html#create/2"},{"type":"function","title":"Membership.Feature.grant/2","doc":"Grant given grant type to a feature.","ref":"Membership.Feature.html#grant/2"},{"type":"function","title":"Examples - Membership.Feature.grant/2","doc":"Function accepts either `Membership.Feature` or `Membership.Plan` grants.\nFunction is merging existing grants with the new ones, so calling grant with same\ngrants will not duplicate entries in table.\n\nTo grant particular feature to a given plan\n\n    iex> Membership.Feature.grant(%Membership.Plan{id: 1}, %Membership.Feature{id: 1})\n\nTo grant particular feature to a given plan\n\n    iex> Membership.Feature.grant(%Membership.Feature{id: 1}, %Membership.Plan{id: 1})","ref":"Membership.Feature.html#grant/2-examples"},{"type":"function","title":"Membership.Feature.grant/3","doc":"","ref":"Membership.Feature.html#grant/3"},{"type":"function","title":"Membership.Feature.load_plan_feature/2","doc":"","ref":"Membership.Feature.html#load_plan_feature/2"},{"type":"function","title":"Membership.Feature.revoke/2","doc":"Revoke given grant type from a member.","ref":"Membership.Feature.html#revoke/2"},{"type":"function","title":"Examples - Membership.Feature.revoke/2","doc":"Function accepts either `Membership.Feature` or `Membership.Plan` grants.\nFunction is directly opposite of `Membership.Member.grant/2`\n\nTo revoke particular feature from a given plan\n\n    iex> Membership.Feature.revoke(%Membership.Plan{id: 1}, %Membership.Feature{id: 1})\n\nTo revoke particular plan from a given feature\n\n    iex> Membership.Feature.revoke(%Membership.Feature{id: 1}, %Membership.Plan{id: 1})","ref":"Membership.Feature.html#revoke/2-examples"},{"type":"function","title":"Membership.Feature.revoke/3","doc":"","ref":"Membership.Feature.html#revoke/3"},{"type":"function","title":"Membership.Feature.table/0","doc":"","ref":"Membership.Feature.html#table/0"},{"type":"type","title":"Membership.Feature.t/0","doc":"A Feature struct","ref":"Membership.Feature.html#t:t/0"},{"type":"module","title":"Membership.Member.Server","doc":"Service\n  this will store the member state","ref":"Membership.Member.Server.html"},{"type":"function","title":"Membership.Member.Server.add_to_calculated_registry/2","doc":"","ref":"Membership.Member.Server.html#add_to_calculated_registry/2"},{"type":"function","title":"Membership.Member.Server.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Membership.Member.Server.html#child_spec/1"},{"type":"function","title":"Membership.Member.Server.fetch_from_calculated_registry/1","doc":"","ref":"Membership.Member.Server.html#fetch_from_calculated_registry/1"},{"type":"function","title":"Membership.Member.Server.fetch_from_calculated_registry/2","doc":"","ref":"Membership.Member.Server.html#fetch_from_calculated_registry/2"},{"type":"function","title":"Membership.Member.Server.show/1","doc":"","ref":"Membership.Member.Server.html#show/1"},{"type":"function","title":"Membership.Member.Server.shutdown/0","doc":"","ref":"Membership.Member.Server.html#shutdown/0"},{"type":"function","title":"Membership.Member.Server.shutdown/1","doc":"","ref":"Membership.Member.Server.html#shutdown/1"},{"type":"function","title":"Membership.Member.Server.start_link/1","doc":"","ref":"Membership.Member.Server.html#start_link/1"},{"type":"module","title":"Membership.MemberFeatures","doc":"MemberFeatures is the association linking the member to the feature you can also set specific features for the membership","ref":"Membership.MemberFeatures.html"},{"type":"function","title":"Membership.MemberFeatures.changeset/2","doc":"","ref":"Membership.MemberFeatures.html#changeset/2"},{"type":"function","title":"Membership.MemberFeatures.create/3","doc":"","ref":"Membership.MemberFeatures.html#create/3"},{"type":"function","title":"Membership.MemberFeatures.table/0","doc":"","ref":"Membership.MemberFeatures.html#table/0"},{"type":"module","title":"Membership.MemberPlans","doc":"MemberPlans is the association linking the member to the plan you can also set specific features for the membership","ref":"Membership.MemberPlans.html"},{"type":"function","title":"Membership.MemberPlans.changeset/2","doc":"","ref":"Membership.MemberPlans.html#changeset/2"},{"type":"function","title":"Membership.MemberPlans.create/3","doc":"","ref":"Membership.MemberPlans.html#create/3"},{"type":"module","title":"Membership.MemberRoles","doc":"MemberFeatures is the association linking the member to the feature you can also set specific features for the membership","ref":"Membership.MemberRoles.html"},{"type":"function","title":"Membership.MemberRoles.changeset/2","doc":"","ref":"Membership.MemberRoles.html#changeset/2"},{"type":"function","title":"Membership.MemberRoles.create/2","doc":"","ref":"Membership.MemberRoles.html#create/2"},{"type":"function","title":"Membership.MemberRoles.table/0","doc":"","ref":"Membership.MemberRoles.html#table/0"},{"type":"module","title":"Membership.Memberships.Supervisor","doc":"Service Supervisor\n  this will supervise the memberships","ref":"Membership.Memberships.Supervisor.html"},{"type":"function","title":"Membership.Memberships.Supervisor.child_spec/0","doc":"","ref":"Membership.Memberships.Supervisor.html#child_spec/0"},{"type":"function","title":"Membership.Memberships.Supervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Membership.Memberships.Supervisor.html#child_spec/1"},{"type":"function","title":"Membership.Memberships.Supervisor.exists?/1","doc":"","ref":"Membership.Memberships.Supervisor.html#exists?/1"},{"type":"function","title":"Membership.Memberships.Supervisor.exists?/2","doc":"","ref":"Membership.Memberships.Supervisor.html#exists?/2"},{"type":"function","title":"Membership.Memberships.Supervisor.find_or_create/1","doc":"","ref":"Membership.Memberships.Supervisor.html#find_or_create/1"},{"type":"function","title":"Membership.Memberships.Supervisor.list/0","doc":"","ref":"Membership.Memberships.Supervisor.html#list/0"},{"type":"function","title":"Membership.Memberships.Supervisor.list/1","doc":"","ref":"Membership.Memberships.Supervisor.html#list/1"},{"type":"function","title":"Membership.Memberships.Supervisor.process_exists?/1","doc":"","ref":"Membership.Memberships.Supervisor.html#process_exists?/1"},{"type":"function","title":"Membership.Memberships.Supervisor.process_exists?/2","doc":"","ref":"Membership.Memberships.Supervisor.html#process_exists?/2"},{"type":"function","title":"Membership.Memberships.Supervisor.reload/1","doc":"","ref":"Membership.Memberships.Supervisor.html#reload/1"},{"type":"function","title":"Membership.Memberships.Supervisor.start/1","doc":"","ref":"Membership.Memberships.Supervisor.html#start/1"},{"type":"function","title":"Membership.Memberships.Supervisor.start_link/0","doc":"","ref":"Membership.Memberships.Supervisor.html#start_link/0"},{"type":"function","title":"Membership.Memberships.Supervisor.start_link/1","doc":"","ref":"Membership.Memberships.Supervisor.html#start_link/1"},{"type":"function","title":"Membership.Memberships.Supervisor.stop/1","doc":"","ref":"Membership.Memberships.Supervisor.html#stop/1"},{"type":"function","title":"Membership.Memberships.Supervisor.stop/2","doc":"","ref":"Membership.Memberships.Supervisor.html#stop/2"},{"type":"function","title":"Membership.Memberships.Supervisor.update/1","doc":"","ref":"Membership.Memberships.Supervisor.html#update/1"},{"type":"function","title":"Membership.Memberships.Supervisor.update_children/0","doc":"","ref":"Membership.Memberships.Supervisor.html#update_children/0"},{"type":"function","title":"Membership.Memberships.Supervisor.whereis/1","doc":"","ref":"Membership.Memberships.Supervisor.html#whereis/1"},{"type":"function","title":"Membership.Memberships.Supervisor.whereis/2","doc":"","ref":"Membership.Memberships.Supervisor.html#whereis/2"},{"type":"module","title":"Membership.Permission.Server","doc":"Service\n  this will store the member state","ref":"Membership.Permission.Server.html"},{"type":"function","title":"Membership.Permission.Server.add/3","doc":"","ref":"Membership.Permission.Server.html#add/3"},{"type":"function","title":"Membership.Permission.Server.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Membership.Permission.Server.html#child_spec/1"},{"type":"function","title":"Membership.Permission.Server.insert/3","doc":"","ref":"Membership.Permission.Server.html#insert/3"},{"type":"function","title":"Membership.Permission.Server.lookup/2","doc":"","ref":"Membership.Permission.Server.html#lookup/2"},{"type":"function","title":"Membership.Permission.Server.start_link/1","doc":"","ref":"Membership.Permission.Server.html#start_link/1"},{"type":"module","title":"Membership.Permissions.Supervisor","doc":"Module Permissions Supervisor\n  this will supervise the permissions","ref":"Membership.Permissions.Supervisor.html"},{"type":"function","title":"Membership.Permissions.Supervisor.child_spec/0","doc":"","ref":"Membership.Permissions.Supervisor.html#child_spec/0"},{"type":"function","title":"Membership.Permissions.Supervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Membership.Permissions.Supervisor.html#child_spec/1"},{"type":"function","title":"Membership.Permissions.Supervisor.exists?/1","doc":"","ref":"Membership.Permissions.Supervisor.html#exists?/1"},{"type":"function","title":"Membership.Permissions.Supervisor.exists?/2","doc":"","ref":"Membership.Permissions.Supervisor.html#exists?/2"},{"type":"function","title":"Membership.Permissions.Supervisor.find_or_create/1","doc":"","ref":"Membership.Permissions.Supervisor.html#find_or_create/1"},{"type":"function","title":"Membership.Permissions.Supervisor.list/0","doc":"","ref":"Membership.Permissions.Supervisor.html#list/0"},{"type":"function","title":"Membership.Permissions.Supervisor.list/1","doc":"","ref":"Membership.Permissions.Supervisor.html#list/1"},{"type":"function","title":"Membership.Permissions.Supervisor.process_exists?/1","doc":"","ref":"Membership.Permissions.Supervisor.html#process_exists?/1"},{"type":"function","title":"Membership.Permissions.Supervisor.process_exists?/2","doc":"","ref":"Membership.Permissions.Supervisor.html#process_exists?/2"},{"type":"function","title":"Membership.Permissions.Supervisor.start/1","doc":"","ref":"Membership.Permissions.Supervisor.html#start/1"},{"type":"function","title":"Membership.Permissions.Supervisor.start_link/0","doc":"","ref":"Membership.Permissions.Supervisor.html#start_link/0"},{"type":"function","title":"Membership.Permissions.Supervisor.start_link/1","doc":"","ref":"Membership.Permissions.Supervisor.html#start_link/1"},{"type":"function","title":"Membership.Permissions.Supervisor.stop/1","doc":"","ref":"Membership.Permissions.Supervisor.html#stop/1"},{"type":"function","title":"Membership.Permissions.Supervisor.stop/2","doc":"","ref":"Membership.Permissions.Supervisor.html#stop/2"},{"type":"function","title":"Membership.Permissions.Supervisor.update_children/0","doc":"","ref":"Membership.Permissions.Supervisor.html#update_children/0"},{"type":"function","title":"Membership.Permissions.Supervisor.whereis/1","doc":"","ref":"Membership.Permissions.Supervisor.html#whereis/1"},{"type":"function","title":"Membership.Permissions.Supervisor.whereis/2","doc":"","ref":"Membership.Permissions.Supervisor.html#whereis/2"},{"type":"module","title":"Membership.Phoenix.Components","doc":"","ref":"Membership.Phoenix.Components.html"},{"type":"module","title":"Membership.Plan.Server","doc":"Plan\n  this will store/fetch the {plan, features} into ets","ref":"Membership.Plan.Server.html"},{"type":"function","title":"Membership.Plan.Server.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Membership.Plan.Server.html#child_spec/1"},{"type":"function","title":"Membership.Plan.Server.reload/0","doc":"","ref":"Membership.Plan.Server.html#reload/0"},{"type":"function","title":"Membership.Plan.Server.start_link/1","doc":"","ref":"Membership.Plan.Server.html#start_link/1"},{"type":"module","title":"Membership.Repo","doc":"Ecto repository","ref":"Membership.Repo.html"},{"type":"function","title":"Membership.Repo.aggregate/3","doc":"","ref":"Membership.Repo.html#aggregate/3"},{"type":"function","title":"Membership.Repo.aggregate/4","doc":"","ref":"Membership.Repo.html#aggregate/4"},{"type":"function","title":"Membership.Repo.all/2","doc":"","ref":"Membership.Repo.html#all/2"},{"type":"function","title":"Membership.Repo.checked_out?/0","doc":"","ref":"Membership.Repo.html#checked_out?/0"},{"type":"function","title":"Membership.Repo.checkout/2","doc":"","ref":"Membership.Repo.html#checkout/2"},{"type":"function","title":"Membership.Repo.child_spec/1","doc":"","ref":"Membership.Repo.html#child_spec/1"},{"type":"function","title":"Membership.Repo.config/0","doc":"","ref":"Membership.Repo.html#config/0"},{"type":"function","title":"Membership.Repo.default_options/1","doc":"","ref":"Membership.Repo.html#default_options/1"},{"type":"function","title":"Membership.Repo.delete/2","doc":"","ref":"Membership.Repo.html#delete/2"},{"type":"function","title":"Membership.Repo.delete!/2","doc":"","ref":"Membership.Repo.html#delete!/2"},{"type":"function","title":"Membership.Repo.delete_all/2","doc":"","ref":"Membership.Repo.html#delete_all/2"},{"type":"function","title":"Membership.Repo.disconnect_all/2","doc":"A convenience function for SQL-based repositories that forces all connections in the\npool to disconnect within the given interval.\n\nSee `Ecto.Adapters.SQL.disconnect_all/3` for more information.","ref":"Membership.Repo.html#disconnect_all/2"},{"type":"function","title":"Membership.Repo.exists?/2","doc":"","ref":"Membership.Repo.html#exists?/2"},{"type":"function","title":"Membership.Repo.explain/3","doc":"A convenience function for SQL-based repositories that executes an EXPLAIN statement or similar\ndepending on the adapter to obtain statistics for the given query.\n\nSee `Ecto.Adapters.SQL.explain/4` for more information.","ref":"Membership.Repo.html#explain/3"},{"type":"function","title":"Membership.Repo.get/3","doc":"","ref":"Membership.Repo.html#get/3"},{"type":"function","title":"Membership.Repo.get!/3","doc":"","ref":"Membership.Repo.html#get!/3"},{"type":"function","title":"Membership.Repo.get_by/3","doc":"","ref":"Membership.Repo.html#get_by/3"},{"type":"function","title":"Membership.Repo.get_by!/3","doc":"","ref":"Membership.Repo.html#get_by!/3"},{"type":"function","title":"Membership.Repo.get_dynamic_repo/0","doc":"","ref":"Membership.Repo.html#get_dynamic_repo/0"},{"type":"function","title":"Membership.Repo.in_transaction?/0","doc":"","ref":"Membership.Repo.html#in_transaction?/0"},{"type":"function","title":"Membership.Repo.insert/2","doc":"","ref":"Membership.Repo.html#insert/2"},{"type":"function","title":"Membership.Repo.insert!/2","doc":"","ref":"Membership.Repo.html#insert!/2"},{"type":"function","title":"Membership.Repo.insert_all/3","doc":"","ref":"Membership.Repo.html#insert_all/3"},{"type":"function","title":"Membership.Repo.insert_or_update/2","doc":"","ref":"Membership.Repo.html#insert_or_update/2"},{"type":"function","title":"Membership.Repo.insert_or_update!/2","doc":"","ref":"Membership.Repo.html#insert_or_update!/2"},{"type":"function","title":"Membership.Repo.load/2","doc":"","ref":"Membership.Repo.html#load/2"},{"type":"function","title":"Membership.Repo.one/2","doc":"","ref":"Membership.Repo.html#one/2"},{"type":"function","title":"Membership.Repo.one!/2","doc":"","ref":"Membership.Repo.html#one!/2"},{"type":"function","title":"Membership.Repo.preload/3","doc":"","ref":"Membership.Repo.html#preload/3"},{"type":"function","title":"Membership.Repo.prepare_query/3","doc":"","ref":"Membership.Repo.html#prepare_query/3"},{"type":"function","title":"Membership.Repo.put_dynamic_repo/1","doc":"","ref":"Membership.Repo.html#put_dynamic_repo/1"},{"type":"function","title":"Membership.Repo.query/3","doc":"A convenience function for SQL-based repositories that executes the given query.\n\nSee `Ecto.Adapters.SQL.query/4` for more information.","ref":"Membership.Repo.html#query/3"},{"type":"function","title":"Membership.Repo.query!/3","doc":"A convenience function for SQL-based repositories that executes the given query.\n\nSee `Ecto.Adapters.SQL.query!/4` for more information.","ref":"Membership.Repo.html#query!/3"},{"type":"function","title":"Membership.Repo.query_many/3","doc":"A convenience function for SQL-based repositories that executes the given multi-result query.\n\nSee `Ecto.Adapters.SQL.query_many/4` for more information.","ref":"Membership.Repo.html#query_many/3"},{"type":"function","title":"Membership.Repo.query_many!/3","doc":"A convenience function for SQL-based repositories that executes the given multi-result query.\n\nSee `Ecto.Adapters.SQL.query_many!/4` for more information.","ref":"Membership.Repo.html#query_many!/3"},{"type":"function","title":"Membership.Repo.reload/2","doc":"","ref":"Membership.Repo.html#reload/2"},{"type":"function","title":"Membership.Repo.reload!/2","doc":"","ref":"Membership.Repo.html#reload!/2"},{"type":"function","title":"Membership.Repo.rollback/1","doc":"","ref":"Membership.Repo.html#rollback/1"},{"type":"function","title":"Membership.Repo.start_link/1","doc":"","ref":"Membership.Repo.html#start_link/1"},{"type":"function","title":"Membership.Repo.stop/1","doc":"","ref":"Membership.Repo.html#stop/1"},{"type":"function","title":"Membership.Repo.stream/2","doc":"","ref":"Membership.Repo.html#stream/2"},{"type":"function","title":"Membership.Repo.to_sql/2","doc":"A convenience function for SQL-based repositories that translates the given query to SQL.\n\nSee `Ecto.Adapters.SQL.to_sql/3` for more information.","ref":"Membership.Repo.html#to_sql/2"},{"type":"function","title":"Membership.Repo.transaction/2","doc":"","ref":"Membership.Repo.html#transaction/2"},{"type":"function","title":"Membership.Repo.truncate/1","doc":"Empty the Database Table","ref":"Membership.Repo.html#truncate/1"},{"type":"function","title":"Membership.Repo.update/2","doc":"","ref":"Membership.Repo.html#update/2"},{"type":"function","title":"Membership.Repo.update!/2","doc":"","ref":"Membership.Repo.html#update!/2"},{"type":"function","title":"Membership.Repo.update_all/3","doc":"","ref":"Membership.Repo.html#update_all/3"},{"type":"module","title":"Membership.Role.Server","doc":"Role\n  this will store/fetch the {role, features} into ets","ref":"Membership.Role.Server.html"},{"type":"function","title":"Membership.Role.Server.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Membership.Role.Server.html#child_spec/1"},{"type":"function","title":"Membership.Role.Server.reload/0","doc":"","ref":"Membership.Role.Server.html#reload/0"},{"type":"function","title":"Membership.Role.Server.start_link/1","doc":"","ref":"Membership.Role.Server.html#start_link/1"},{"type":"task","title":"mix membership","doc":"","ref":"Mix.Tasks.Membership.html"},{"type":"function","title":"Mix.Tasks.Membership.run/1","doc":"","ref":"Mix.Tasks.Membership.html#run/1"},{"type":"task","title":"mix membership.components","doc":"After configuring your default ecto repo in `:ecto_repos`\n  Run mix membership.components to generates a `membership_components` component,\n  which has basic phoenix membership components.","ref":"Mix.Tasks.Membership.Components.html"},{"type":"function","title":"Mix.Tasks.Membership.Components.run/1","doc":"","ref":"Mix.Tasks.Membership.Components.html#run/1"},{"type":"task","title":"mix membership.install","doc":"After configuring your default ecto repo in `:ecto_repos`\n  Run mix membership.install to generates a `setup_membership_tables` migration,\n  which Membership tables, as well as required indexes.","ref":"Mix.Tasks.Membership.Install.html"},{"type":"function","title":"Mix.Tasks.Membership.Install.run/1","doc":"","ref":"Mix.Tasks.Membership.Install.html#run/1"},{"type":"task","title":"mix membership.setup","doc":"","ref":"Mix.Tasks.Membership.Setup.html"},{"type":"function","title":"Mix.Tasks.Membership.Setup.run/1","doc":"","ref":"Mix.Tasks.Membership.Setup.html#run/1"},{"type":"module","title":"Post","doc":"","ref":"Post.html"},{"type":"macro","title":"Post.as_authorized/3","doc":"Macro for wrapping protected code","ref":"Post.html#as_authorized/3"},{"type":"macro","title":"Example - Post.as_authorized/3","doc":"defmodule HelloTest do\n      use Membership\n      member = HelloTest.Repo.get(Membership.Member, 1)\n      {:ok, member }  = load_and_authorize_member(member)\n\n      def test_authorization do\n        as_authorized(member, :test_authorization) do\n          IO.inspect(\"This code is executed only for authorized member\")\n        end\n      end\n    end","ref":"Post.html#as_authorized/3-example"},{"type":"function","title":"Post.authorized?/2","doc":"Returns authorization result on collected member and required features/plans","ref":"Post.html#authorized?/2"},{"type":"function","title":"Example - Post.authorized?/2","doc":"defmodule HelloTest do\n      use Membership\n\n      def test_authorization do\n        case authorized? do\n          :ok -> \"Member is authorized\"\n          {:error, message: _message} -> \"Member is not authorized\"\n      end\n    end\n    end","ref":"Post.html#authorized?/2-example"},{"type":"macro","title":"Post.calculated/2","doc":"Defines calculated permission to be evaluated in runtime","ref":"Post.html#calculated/2"},{"type":"macro","title":"Examples - Post.calculated/2","doc":"defmodule HelloTest do\n      use Membership\n      member = HelloTest.Repo.get(Membership.Member, 1)\n      {:ok, member }  = load_and_authorize_member(member)\n\n      def test_authorization do\n        permissions do\n          calculated(fn member ->\n            member.email_confirmed?\n          end)\n        end\n\n        as_authorized(member) do\n          IO.inspect(\"This code is executed only for authorized member\")\n        end\n      end\n    end\n\nYou can also use DSL form which takes function name as argument\n\n      defmodule HelloTest do\n      use Membership\n\n      def test_authorization do\n      use Membership\n      member = HelloTest.Repo.get(Membership.Member, 1)\n     {:ok, member } = load_and_authorize_member(member)\n\n        permissions do\n          calculated(member,:email_confirmed)\n        end\n\n        as_authorized(member) do\n          IO.inspect(\"This code is executed only for authorized member\")\n        end\n      end\n\n      def email_confirmed(member) do\n        member.email_confirmed?\n      end\n    end\n\n  For more complex calculation you need to pass bindings to the function\n\n      defmodule HelloTest do\n      use Membership\n      member = HelloTest.Repo.get(Membership.Member, 1)\n      {:ok, member} = load_and_authorize_member(member)\n\n      def test_authorization do\n        post = %Post{owner_id: 1}\n\n        permissions do\n          calculated(member,:is_owner, [post])\n          calculated(fn member, [post] ->\n            post.owner_id == member.id\n          end)\n        end\n\n        as_authorized(member) do\n          IO.inspect(\"This code is executed only for authorized member\")\n        end\n      end\n\n      def is_owner(member, [post]) do\n        post.owner_id == member.id\n      end\n    end","ref":"Post.html#calculated/2-examples"},{"type":"macro","title":"Post.calculated/3","doc":"","ref":"Post.html#calculated/3"},{"type":"macro","title":"Post.calculated/4","doc":"","ref":"Post.html#calculated/4"},{"type":"function","title":"Post.calculated_function/2","doc":"","ref":"Post.html#calculated_function/2"},{"type":"function","title":"Post.calculated_macro/2","doc":"","ref":"Post.html#calculated_macro/2"},{"type":"function","title":"Post.confirmed_email/1","doc":"","ref":"Post.html#confirmed_email/1"},{"type":"function","title":"Post.delete_post/3","doc":"","ref":"Post.html#delete_post/3"},{"type":"function","title":"Post.has_feature/2","doc":"Requires a feature within permissions block","ref":"Post.html#has_feature/2"},{"type":"function","title":"Example - Post.has_feature/2","doc":"defmodule HelloTest do\n      use Membership\n\n      def test_authorization do\n        permissions do\n          has_feature(:admin)\n        end\n      end\n    end","ref":"Post.html#has_feature/2-example"},{"type":"function","title":"Post.has_feature?/2","doc":"Perform feature check on passed member and feature","ref":"Post.html#has_feature?/2"},{"type":"function","title":"Post.has_feature?/3","doc":"","ref":"Post.html#has_feature?/3"},{"type":"function","title":"Post.has_plan/2","doc":"Requires an plan within permissions block","ref":"Post.html#has_plan/2"},{"type":"function","title":"Example - Post.has_plan/2","doc":"defmodule HelloTest do\n      use Membership\n\n      def test_authorization do\n        permissions do\n          has_plan(:gold, :test_authorization)\n        end\n      end\n    end","ref":"Post.html#has_plan/2-example"},{"type":"function","title":"Post.has_plan?/3","doc":"Perform authorization on passed member and plans","ref":"Post.html#has_plan?/3"},{"type":"function","title":"Post.has_role/2","doc":"Requires an role within permissions block","ref":"Post.html#has_role/2"},{"type":"function","title":"Example - Post.has_role/2","doc":"defmodule HelloTest do\n      use Membership\n\n      def test_authorization do\n        permissions do\n          has_role(:gold, :test_authorization)\n        end\n      end\n    end","ref":"Post.html#has_role/2-example"},{"type":"function","title":"Post.has_role?/3","doc":"Perform authorization on passed member and roles","ref":"Post.html#has_role?/3"},{"type":"function","title":"Post.ignored_functions/0","doc":"The Function list to ignore when building the permissions registry's","ref":"Post.html#ignored_functions/0"},{"type":"function","title":"Post.load_and_authorize_member/1","doc":"","ref":"Post.html#load_and_authorize_member/1"},{"type":"function","title":"Post.load_and_authorize_member/2","doc":"","ref":"Post.html#load_and_authorize_member/2"},{"type":"function","title":"Post.load_ets_data/1","doc":"Load the plans into ets for the module/functions","ref":"Post.html#load_ets_data/1"},{"type":"function","title":"Post.no_macro/2","doc":"","ref":"Post.html#no_macro/2"},{"type":"function","title":"Post.no_permissions/2","doc":"","ref":"Post.html#no_permissions/2"},{"type":"macro","title":"Post.permissions/1","doc":"Macro for defining required permissions","ref":"Post.html#permissions/1"},{"type":"macro","title":"Example - Post.permissions/1","doc":"defmodule HelloTest do\n      use Membership\n\n      def test_authorization do\n        permissions do\n          has_feature(:admin_feature, :test_authorization)\n          has_plan(:gold, :test_authorization)\n        end\n      end\n    end","ref":"Post.html#permissions/1-example"},{"type":"macro","title":"Post.permissions/2","doc":"","ref":"Post.html#permissions/2"},{"type":"function","title":"Post.update/3","doc":"","ref":"Post.html#update/3"},{"type":"module","title":"Membership.Member","doc":"Member is a main actor for determining features","ref":"Membership.Member.html"},{"type":"function","title":"Membership.Member.build/1","doc":"","ref":"Membership.Member.html#build/1"},{"type":"function","title":"Membership.Member.changeset/2","doc":"","ref":"Membership.Member.html#changeset/2"},{"type":"function","title":"Membership.Member.create/1","doc":"","ref":"Membership.Member.html#create/1"},{"type":"function","title":"Membership.Member.fetch_removed_features/1","doc":"","ref":"Membership.Member.html#fetch_removed_features/1"},{"type":"function","title":"Membership.Member.generate_identifier/1","doc":"","ref":"Membership.Member.html#generate_identifier/1"},{"type":"function","title":"Membership.Member.grant/2","doc":"Grant given grant type to a member.","ref":"Membership.Member.html#grant/2"},{"type":"function","title":"Examples - Membership.Member.grant/2","doc":"Function accepts either `Membership.Feature` or `Membership.Plan` grants.\nFunction is merging existing grants with the new ones, so calling grant with same\ngrants will not duplicate entries in table.\n\nTo grant particular feature to a given member\n\n    iex> Membership.Member.grant(%Membership.Member{id: 1}, %Membership.Feature{id: 1})\n\nTo grant particular plan to a given member\n\n    iex> Membership.Member.grant(%Membership.Member{id: 1}, %Membership.Plan{id: 1})","ref":"Membership.Member.html#grant/2-examples"},{"type":"function","title":"Membership.Member.grant/3","doc":"","ref":"Membership.Member.html#grant/3"},{"type":"function","title":"Membership.Member.load_member_feature/2","doc":"","ref":"Membership.Member.html#load_member_feature/2"},{"type":"function","title":"Membership.Member.load_member_plan/2","doc":"","ref":"Membership.Member.html#load_member_plan/2"},{"type":"function","title":"Membership.Member.load_member_role/2","doc":"","ref":"Membership.Member.html#load_member_role/2"},{"type":"function","title":"Membership.Member.revoke/2","doc":"Revoke given grant type from a member.","ref":"Membership.Member.html#revoke/2"},{"type":"function","title":"Examples - Membership.Member.revoke/2","doc":"Function accepts either `Membership.Feature` or `Membership.Plan` grants.\nFunction is directly opposite of `Membership.Member.grant/2`\n\nTo revoke particular feature from a given member\n\n    iex> Membership.Member.revoke(%Membership.Member{id: 1}, %Membership.Feature{id: 1})\n\nTo revoke particular plan from a given member\n\n    iex> Membership.Member.revoke(%Membership.Member{id: 1}, %Membership.Plan{id: 1})","ref":"Membership.Member.html#revoke/2-examples"},{"type":"function","title":"Membership.Member.revoke/3","doc":"","ref":"Membership.Member.html#revoke/3"},{"type":"function","title":"Membership.Member.sync_features/1","doc":"Sync features column with the member features and member plans pivot tables.\nwe do this for caching reasons, ie holding the plan[feature] and extra feature identifiers summed\ninto a list and stored in features column of the member, we  query this to see if member has\nex feature vs repo lookup by plan and checking if plan has said feature","ref":"Membership.Member.html#sync_features/1"},{"type":"function","title":"Membership.Member.table/0","doc":"","ref":"Membership.Member.html#table/0"},{"type":"type","title":"Membership.Member.t/0","doc":"A member struct","ref":"Membership.Member.html#t:t/0"},{"type":"module","title":"Membership.Plan","doc":"Plan is main representation of a single plan","ref":"Membership.Plan.html"},{"type":"function","title":"Membership.Plan.all/0","doc":"","ref":"Membership.Plan.html#all/0"},{"type":"function","title":"Membership.Plan.build/3","doc":"","ref":"Membership.Plan.html#build/3"},{"type":"function","title":"Membership.Plan.changeset/2","doc":"","ref":"Membership.Plan.html#changeset/2"},{"type":"function","title":"Membership.Plan.create/1","doc":"","ref":"Membership.Plan.html#create/1"},{"type":"function","title":"Membership.Plan.create/3","doc":"","ref":"Membership.Plan.html#create/3"},{"type":"function","title":"Membership.Plan.grant/2","doc":"Grant given grant type to a feature.","ref":"Membership.Plan.html#grant/2"},{"type":"function","title":"Examples - Membership.Plan.grant/2","doc":"Function accepts either `Membership.Plan` or `Membership.Feature` grants.\nFunction is merging existing grants with the new ones, so calling grant with same\ngrants will not duplicate entries in table.\n\nTo grant particular feature to a given plan\n\n    iex> Membership.Plan.grant(%Membership.Feature{id: 1}, %Membership.Plan{id: 1})\n\nTo grant particular feature to a given plan\n\n    iex> Membership.Plan.grant(%Membership.Plan{id: 1}, %Membership.Feature{id: 1})","ref":"Membership.Plan.html#grant/2-examples"},{"type":"function","title":"Membership.Plan.grant/3","doc":"","ref":"Membership.Plan.html#grant/3"},{"type":"function","title":"Membership.Plan.load_plan_feature/2","doc":"","ref":"Membership.Plan.html#load_plan_feature/2"},{"type":"function","title":"Membership.Plan.revoke/2","doc":"Revoke given grant type from a member.","ref":"Membership.Plan.html#revoke/2"},{"type":"function","title":"Examples - Membership.Plan.revoke/2","doc":"Function accepts either `Membership.Plan` or `Membership.Feature` grants.\nFunction is directly opposite of `Membership.Member.grant/2`\n\nTo revoke particular feature from a given plan\n\n    iex> Membership.Plan.revoke(%Membership.Feature{id: 1}, %Membership.Plan{id: 1})\n\nTo revoke particular plan from a given feature\n\n    iex> Membership.Plan.revoke(%Membership.Plan{id: 1}, %Membership.Feature{id: 1})","ref":"Membership.Plan.html#revoke/2-examples"},{"type":"function","title":"Membership.Plan.revoke/3","doc":"","ref":"Membership.Plan.html#revoke/3"},{"type":"function","title":"Membership.Plan.table/0","doc":"","ref":"Membership.Plan.html#table/0"},{"type":"type","title":"Membership.Plan.t/0","doc":"A plan struct","ref":"Membership.Plan.html#t:t/0"},{"type":"module","title":"Membership.Role","doc":"Role is main representation of feature flags assigned to a role","ref":"Membership.Role.html"},{"type":"function","title":"Membership.Role.all/0","doc":"","ref":"Membership.Role.html#all/0"},{"type":"function","title":"Membership.Role.build/2","doc":"","ref":"Membership.Role.html#build/2"},{"type":"function","title":"Membership.Role.changeset/2","doc":"","ref":"Membership.Role.html#changeset/2"},{"type":"function","title":"Membership.Role.create/3","doc":"","ref":"Membership.Role.html#create/3"},{"type":"function","title":"Membership.Role.grant/2","doc":"Grant given grant type to a feature.","ref":"Membership.Role.html#grant/2"},{"type":"function","title":"Examples - Membership.Role.grant/2","doc":"Function accepts either `Membership.Role` or `Membership.Feature` grants.\nFunction is merging existing grants with the new ones, so calling grant with same\ngrants will not duplicate entries in table.\n\nTo grant particular feature to a given role\n\n    iex> Membership.Role.grant(%Membership.Feature{id: 1}, %Membership.Role{id: 1})\n\nTo grant particular feature to a given role\n\n    iex> Membership.Role.grant(%Membership.Role{id: 1}, %Membership.Feature{id: 1})","ref":"Membership.Role.html#grant/2-examples"},{"type":"function","title":"Membership.Role.grant/3","doc":"","ref":"Membership.Role.html#grant/3"},{"type":"function","title":"Membership.Role.load_role_feature/2","doc":"","ref":"Membership.Role.html#load_role_feature/2"},{"type":"function","title":"Membership.Role.revoke/2","doc":"Revoke given grant type from a member.","ref":"Membership.Role.html#revoke/2"},{"type":"function","title":"Examples - Membership.Role.revoke/2","doc":"Function accepts either `Membership.Role` or `Membership.Feature` grants.\nFunction is directly opposite of `Membership.Member.grant/2`\n\nTo revoke particular feature from a given plan\n\n    iex> Membership.Role.revoke(%Membership.Feature{id: 1}, %Membership.Role{id: 1})\n\nTo revoke particular plan from a given feature\n\n    iex> Membership.Role.revoke(%Membership.Role{id: 1}, %Membership.Feature{id: 1})","ref":"Membership.Role.html#revoke/2-examples"},{"type":"function","title":"Membership.Role.revoke/3","doc":"","ref":"Membership.Role.html#revoke/3"},{"type":"function","title":"Membership.Role.table/0","doc":"","ref":"Membership.Role.html#table/0"},{"type":"type","title":"Membership.Role.t/0","doc":"A Role struct","ref":"Membership.Role.html#t:t/0"},{"type":"extras","title":"🛡 Membership 🛡","doc":"# 🛡 Membership 🛡\n\n[![Coverage Status](https://coveralls.io/repos/github/mithereal/ex_membership/badge.svg?branch=main)](https://coveralls.io/github/mithereal/ex_membership?branch=main)\n![CircleCI](https://img.shields.io/circleci/build/github/mithereal/ex_membership)\n[![Version](https://img.shields.io/hexpm/v/ex_membership.svg?style=flat-square)](https://hex.pm/packages/ex_membership)\n![GitHub](https://img.shields.io/github/license/mithereal/ex_membership)\n![GitHub last commit (branch)](https://img.shields.io/github/last-commit/mithereal/ex_membership/main)\n\nMembership is toolkit for granular feature management for members. It allows you to define granular features such\nas: [:can_edit, :can_delete] on a per module basis\neach module has an ets backed registry with {function, permission} tuple.\nthis allows us to have plans with multiple features which members can subscribe to\nwe then can hold each user in a registry and compare features on a function level.\n\nHere is a small example:\n\n```elixir\ndefmodule Post do\n  use Membership, registry: :post\n  \n  alias Post \n  alias Membership.Repo \n  alias Membership.Member\n  \n   def create_post(id, member_id \\\\ 1) do\n    member = Repo.get(Member, member_id)\n    post = %Post{id: id}\n\n    permissions(member) do\n      has_plan(:editor)\n    end\n\n    as_authorized(member) do\n      Repo.get(Post, id) |> Repo.insert_or_update()\n    end\n\n    # Notice that you can use both macros or functions\n\n    case authorized? do\n      :ok -> Repo.get(Post, id) |> Repo.delete()\n      {:error, message} -> raise message\n      _ -> raise \"Member is not authorized\"\n    end\n  end\n\n  def delete_post(id, member_id \\\\ 1) do\n    member = Repo.get(Member, member_id)\n    member = load_and_authorize_member(member)\n    post = %Post{id: id}\n\n    permissions do\n      has_plan(:admin) # or\n      has_plan(:editor) # or\n      has_feature(:delete_posts) # or\n      has_feature(:delete, post) # Entity related features\n      calculated(fn member ->\n        Post.email_confirmed?(member)\n      end)\n    end\n\n    as_authorized(member) do\n      Repo.get(Post, id) |> Repo.delete()\n    end\n\n    # Notice that you can use both macros or functions\n\n    case authorized? do\n      :ok -> Repo.get(Post, id) |> Repo.delete()\n      {:error, message} -> \"Raise error\"\n      _ -> \"Raise error\"\n    end\n  end\n  end\n\n```","ref":"readme.html"},{"type":"extras","title":"Mix Tasks - 🛡 Membership 🛡","doc":"To create the migrations in your elixir project run\n\n```bash\nmix membership.install\n```","ref":"readme.html#mix-tasks"},{"type":"extras","title":"Features - 🛡 Membership 🛡","doc":"- [x] `Member` -> `[Feature]` permission schema\n- [x] `Plan` -> `[Feature]` permission schema\n- [x] `Role` -> `[Feature]` permission schema\n- [x] `Member` -> `[Plan]` -> `[Feature]` permission schema\n- [x] `Member` -> `[Role]` -> `[Feature]` permission schema\n- [] `Member` -> `Object` -> `[Feature]` permission schema\n- [x] Computed permission in runtime\n- [x] Easily readable DSL\n- [ ] [ueberauth](https://github.com/ueberauth/ueberauth) integration\n- [ ] [absinthe](https://github.com/absinthe-graphql/absinthe) middleware\n- [ ] Session plug to get current_user","ref":"readme.html#features"},{"type":"extras","title":"Installation - 🛡 Membership 🛡","doc":"```elixir\ndef deps do\n  [\n    {:ex_membership, \">= 0.0.0\"}\n  ]\nend\n```\n\n```elixir\n# In your config/config.exs file\nconfig :ex_membership, Membership.Repo,\n  username: \"postgres\",\n  password: \"postgres\",\n  database: \"membership_dev\",\n  hostname: \"localhost\"\n```\n\n```elixir\niex> mix membership.setup\n```\n\n#","ref":"readme.html#installation"},{"type":"extras","title":"Usage with ecto - 🛡 Membership 🛡","doc":"Membership is originally designed to be used with Ecto. Usually you will want to have your own table\nfor `Accounts`/`Users` living in your application. To do so you can link member with `belongs_to` association within\nyour schema.\n\n```elixir\n# In your migrations add member_id field\ndefmodule Sample.Migrations.CreateUsersTable do\n  use Ecto.Migration\n\n  def change do\n    create table(:users) do\n      add :username, :string\n      add :member_id, references(Membership.Member.table())\n\n      timestamps()\n    end\n\n    create unique_index(:users, [:username])\n  end\nend\n\n```\n\nThis will allow you link any internal entity with 1-1 association to members. Please note that you need to create member\non each user creation (e.g with `Membership.Member.changeset/2`) and call `put_assoc` inside your changeset\n\n```elixir\n# In schema defintion\ndefmodule Sample.User do\n  use Ecto.Schema\n\n  schema \"users\" do\n    field :username, :String\n\n    belongs_to :member, Membership.Member\n\n    timestamps()\n  end\nend\n```\n\n```elixir\n# In your model\ndefmodule Sample.Post do\n  use Membership, registry: :post\n\n  def delete_post(id, member_id) do\n    user = Sample.Repo.get(Sample.User, member_id)\n    load_and_authorize_member(user)\n    # Function allows multiple signatues of member it can\n    # be either:\n    #  * %Membership.Member{}\n    #  * %AnyStruct{member: %Membership.Member{}}\n    #  * %AnyStruct{member_id: id} (this will perform database preload)\n\n\n    permissions do\n      has_plan(:admin) # or\n      has_plan(:editor) # or\n      has_feature(:delete_posts) # or\n    end\n\n    member_authorized do\n      Sample.Repo.get(Sample.Post, id) |> Sample.repo.delete()\n    end\n\n    # Notice that you can use both macros or functions\n\n    case authorized? do\n      :ok -> Sample.Repo.get(Sample.Post, id) |> Sample.repo.delete()\n      {:error, message} -> raise message\n      _ -> raise \"Member is not authorized\"\n    end\n  end\n  end\n\n```\n\nMembership tries to infer the member, so it is easy to pass any struct (could be for example `User` in your application)\nwhich has set up `belongs_to` association for member. If the member was already preloaded from database Membership will\ntake it as loaded member. If you didn't do preload and just loaded `User` -> `Repo.get(User, 1)` Membership will fetch\nthe member on each authorization try.\n\n#","ref":"readme.html#usage-with-ecto"},{"type":"extras","title":"Calculated permissions - 🛡 Membership 🛡","doc":"Often you will come to case when `static` permissions are not enough. For example allow only users who confirmed their\nemail address.\n\n```elixir\ndefmodule Sample.Post do\n use Membership, registry: :post\n \n  def create(id \\\\ 1) do\n    member = Sample.Repo.get(Sample.User, id)\n    load_and_authorize_member(member)\n\n    permissions(member) do\n          calculated(\n        member,\n        fn member ->\n          Post.confirmed_email(member)\n        end,\n        :create_calculated\n      )\n    end\n    end\n    end\n```\n\nWe can also use DSL form of `calculated` keyword\n\n```elixir\ndefmodule Sample.Post do\n use Membership, registry: :post\n \n  def create(id \\\\ 1) do\n    member = Sample.Repo.get(Sample.User, id)\n    load_and_authorize_member(member)\n \n      permissions(member) do\n          calculated(\n        member,\n        :confirmed_email,\n        :create_calculated\n      )\n    end\n\n\n  def confirmed_email(member) do\n    member.email_confirmed?\n  end\nend\nend\n```\n\n#","ref":"readme.html#calculated-permissions"},{"type":"extras","title":"Composing calculations - 🛡 Membership 🛡","doc":"When we need to member calculation based on external data we can invoke bindings to `calculated/2`\n\n```elixir\ndefmodule Sample.Post do\n use Membership, registry: :post\n \n  def create(id \\\\ 1) do\n    member = Sample.Repo.get(Sample.User, id)\n    load_and_authorize_member(member)\n    post = %Post{owner_id: member.id}\n\n    permissions(member) do\n      calculated(member,:confirmed_email)\n      calculated(member, :is_owner, [post])\n    end\n  end\n\n  def confirmed_email(member) do\n    member.email_confirmed?\n  end\n\n  def is_owner(member, [post]) do\n    member.id == post.owner_id\n  end\nend\n```\n\nTo perform exclusive features such as `when User is owner of post AND is in editor plan` we can do so as in following\nexample\n\n```elixir\ndefmodule Sample.Post do\n use Membership, registry: :post\n \n  def create(member_id \\\\ 1) do\n    member = Sample.Repo.get(Sample.User, member_id)\n    load_and_authorize_member(member)\n    post = %Post{owner_id: member.id}\n\n    permissions do\n      has_plan(:editor)\n    end\n\n    member_authorized do\n      case is_owner(member, post) do\n        :ok -> {:ok, \"Member is the Owner of Post\"}\n        {:error, message} -> {:error, message}\n      end\n    end\n  end\n\n  def is_owner(member, post) do\n    load_and_authorize_member(member)\n\n    permissions do\n      calculated(fn p, [post] ->\n        p.id == post.owner_id\n      end)\n    end\n\n    authorized?\n  end\nend\n```\n\nWe can simplify example in this case by excluding DSL for permissions\n\n```elixir\ndefmodule Sample.Post do\n use Membership, registry: :post\n \n  def create(id \\\\ 1 , member_id \\\\ 1) do\n    member = Sample.Repo.get(Sample.User, member_id)\n    load_and_authorize_member(member)\n    post = %Post{owner_id: member.id}\n\n    # We can also use has_feature?/2\n    if has_plan?(member, :admin) and is_owner(member, post) do\n      {:ok, \"Member Can Modify Post\"}\n    end\n  end\n\n  def is_owner(member, post) do\n    member.id == post.owner_id\n  end\nend\n```\n\n#","ref":"readme.html#composing-calculations"},{"type":"extras","title":"Member related features - 🛡 Membership 🛡","doc":"#","ref":"readme.html#member-related-features"},{"type":"extras","title":"Granting features - 🛡 Membership 🛡","doc":"Let's assume we want to create new `Plan` - _gold_ which is able to delete accounts inside our system. We want to have\nspecial `Member` who is given this _plan_ but also he is able to have `Feature` for banning users.\n\n1. Create member\n\n```elixir\niex> {:ok, member} = %Membership.Member{} |> Membership.Repo.insert()\n```\n\n2. Create some features\n\n```elixir\niex> {:ok, feature_delete} = Membership.Feature.build(\"delete_accounts\", \"Delete accounts of users\") |> Membership.Repo.insert()\niex> {:ok, feature_ban} = Membership.Feature.build(\"ban_accounts\", \"Ban users\") |> Membership.Repo.insert()\n```\n\n3. Create plan\n\n```elixir\niex> {:ok, plan} = Membership.Plan.build(\"gold\", [], \"Gold Package\") |> Membership.Repo.insert()\n```\n\n4. Grant features to a plan\n\n```elixir\niex> Membership.Plan.grant(plan, feature_delete)\n```\n\n5. Grant plan to a member\n\n```elixir\niex> Membership.Member.grant(member, plan)\n```\n\n6. Grant features to a member\n\n```elixir\niex> Membership.Member.grant(member, feature_ban)\n```\n\n```elixir\niex> member |> Membership.Repo.preload([:plan_memberships, :extra_features])\n%Membership.Member{\nfeatures: [\"ban_accounts\"],\nidentifier: \"asfdcxfdsr42424eq2\",\n  plan_memberships: [\n    %Membership.Plan{\n      identifier: \"gold\"\n      features: [\"delete_accounts\"]\n    }\n  ]\n}\n```\n\n#","ref":"readme.html#granting-features"},{"type":"extras","title":"Revoking features - 🛡 Membership 🛡","doc":"Same as we can grant any features to models we can also revoke them.\n\n```elixir\niex> Membership.Member.revoke(member, plan)\niex> member |> Membership.Repo.preload([:plan_memberships, :extra_features])\n%Membership.Member{\nfeatures: [],\nidentifier: \"asfdcxfdsr42424eq2\",\n  plan_memberships: []\n}\niex> Membership.Member.revoke(member, feature_ban)\niex> member |> Membership.Repo.preload([:plan_memberships, :extra_features])\n%Membership.Member{\nfeatures: [],\nidentifier: \"asfdcxfdsr42424eq2\",\n  plan_memberships: []\n}\n```","ref":"readme.html#revoking-features"},{"type":"extras","title":"License - 🛡 Membership 🛡","doc":"[MIT © Jason Clark](mailto:mithereal@gmail.com)","ref":"readme.html#license"}],"content_type":"text/markdown"}